@@ -1,9 +1,10 @@
+"""Main module"""
 import atexit
 from .compile import Logger
 from .terminal.utils import RestartException, handle_exception
-from .terminal import GlobalData, Colors, handle_exception, start
+from .terminal import GlobalData, Colors, start
 
-VERSION = 'v1.2.6'
+VERSION = 'v1.2.8'
 CONFIG_FILE_NAME = 'jmc_config.json'
 GlobalData().init(VERSION, CONFIG_FILE_NAME)
 
@@ -13,7 +14,8 @@ logger = Logger(__name__)
 
 
 def main():
-    atexit.register(lambda: print(Colors.EXIT.value, end=""))
+    """Main function"""
+    atexit.register(lambda: print(Colors.EXIT.value))
     logger.info("Starting session")
     global_data = GlobalData()
     while True:
