@@ -3,7 +3,7 @@ from ...tokenizer import Token, TokenType
 from ...exception import JMCSyntaxException, JMCMissingValueError, JMCValueError
 from ...datapack_data import Item
 from ...datapack import DataPack
-from ..utils import ArgType, PlayerType, ScoreboardPlayer, FormattedText
+from ..utils import ArgType, NumberType, PlayerType, ScoreboardPlayer, FormattedText
 from ..jmc_function import JMCFunction, FuncType, func_property
 from .._flow_control import parse_switch
 
@@ -67,7 +67,7 @@ class RightClickSetup(JMCFunction):
         if self.is_never_used():
             self.datapack.add_tick_command(
                 f"""execute as @a[scores={{{self.rc_obj}=1..}}] at @s run {self.datapack.add_raw_private_function(self.name,
-                                                       [f'scoreboard players reset @s {self.rc_obj}'], 'main')}""")
+                                                       [f'scoreboard players set @s {self.rc_obj} 0'], 'main')}""")
 
         main_func = self.get_private_function('main')
 
@@ -154,7 +154,7 @@ class ItemCreate(JMCFunction):
                 self.datapack.add_tick_command(
                     f"""execute as @a[scores={{{self.rc_obj}=1..}}] at @s run {self.datapack.add_raw_private_function(self.name,
                                                        [
-                                                        f'scoreboard players reset @s {self.rc_obj}',
+                                                        f'scoreboard players set @s {self.rc_obj} 0',
                                                         f"execute store result score {self.tag_id_var} {DataPack.var_name} run data get entity @s SelectedItem.tag.{self.id_name}",
                                                         f"execute if score {self.tag_id_var} {DataPack.var_name} matches 1.. run {self.datapack.call_func(self.name, 'found')}"
                                                         ], 'main')}""")
@@ -185,11 +185,11 @@ class ItemCreate(JMCFunction):
                 self.token,
                 self.tokenizer)
 
-        lore_ = ",".join([repr(str(FormattedText(lore, self.token, self.tokenizer, is_default_no_italic=True)))
+        lore_ = ",".join([repr(str(FormattedText(lore, self.raw_args["lore"].token, self.tokenizer, self.datapack, is_default_no_italic=True)))
                          for lore in lores])
 
         nbt["display"] = Token.empty(f"""{{Name:{repr(
-            str(FormattedText(name, self.token, self.tokenizer, is_default_no_italic=True))
+            str(FormattedText(name, self.raw_args["displayName"].token, self.tokenizer, self.datapack ,is_default_no_italic=True))
             )},Lore:[{lore_}]}}""")
 
         self.datapack.data.item[self.args["itemId"]] = Item(
@@ -200,6 +200,95 @@ class ItemCreate(JMCFunction):
         return ""
 
 
+@func_property(
+    func_type=FuncType.LOAD_ONLY,
+    call_string='Item.createSign',
+    arg_type={
+        "itemId": ArgType.KEYWORD,
+        "variant": ArgType.KEYWORD,
+        "displayName": ArgType.STRING,
+        "lore": ArgType.LIST,
+        "texts": ArgType.LIST,
+        "nbt": ArgType.JS_OBJECT,
+        "onClick": ArgType.FUNC
+    },
+    name='item_create_sign',
+    defaults={
+        "lore": "",
+        "texts": "",
+        "nbt": "",
+        "onClick": ""
+    }
+)
+class ItemCreateSign(JMCFunction):
+    _variants = {'oak', 'spruce', 'birch', 'jungle',
+                 'acacia', 'dark_oak', 'crimson', 'warped'}
+
+    def call(self) -> str:
+        variant = self.args["variant"]
+
+        if variant not in self._variants:
+            raise JMCValueError(
+                f"Unrecognized wood variant for sign ({variant})",
+                self.raw_args["variant"].token,
+                self.tokenizer,
+                suggestion=f"Available variants are {' '.join(repr(i) for i in self._variants)}")
+
+        on_click = self.args["onClick"]
+        name = self.args["displayName"]
+        if self.args["lore"]:
+            lores = self.datapack.parse_list(
+                self.raw_args["lore"].token, self.tokenizer, TokenType.STRING)
+        else:
+            lores = []
+
+        if self.args["texts"]:
+            texts = self.datapack.parse_list(
+                self.raw_args["texts"].token, self.tokenizer, TokenType.STRING)
+        else:
+            texts = []
+
+        if len(texts) > 4:
+            raise JMCValueError(
+                f"Sign may only have 4 lines of text (got {len(texts)})",
+                self.raw_args["texts"].token,
+                self.tokenizer)
+        texts.extend([''] * (4 - len(texts)))
+
+        nbt = self.tokenizer.parse_js_obj(
+            self.raw_args["nbt"].token) if self.args["nbt"] else {}
+
+        if "display" in nbt:
+            raise JMCValueError(
+                "display is already inside the nbt",
+                self.token,
+                self.tokenizer)
+
+        lore_ = ",".join(repr(str(FormattedText(lore, self.raw_args["lore"].token, self.tokenizer, self.datapack, is_default_no_italic=True)))
+                         for lore in lores)
+        formatted_texts_ = [FormattedText(text, self.raw_args["texts"].token, self.tokenizer, self.datapack)
+                            for text in texts]
+        if on_click:
+            formatted_texts_[0].add_key(
+                "clickEvent", {
+                    "action": "run_command", "value": on_click})
+            if not formatted_texts_[0]:
+                formatted_texts_[0].add_key("text", "")
+
+        texts_ = [repr(str(text)) for text in formatted_texts_]
+
+        nbt["display"] = Token.empty(f"""{{Name:{repr(
+            str(FormattedText(name, self.raw_args["displayName"].token, self.tokenizer, self.datapack, is_default_no_italic=True))
+            )},Lore:[{lore_}]}},BlockEntityTag:{{Text1:{texts_[0]},Text2:{texts_[1]},Text3:{texts_[2]},Text4:{texts_[3]}}}""")
+
+        self.datapack.data.item[self.args["itemId"]] = Item(
+            variant + '_sign',
+            self.datapack.token_dict_to_raw_js_object(nbt, self.tokenizer),
+        )
+
+        return ""
+
+
 @func_property(
     func_type=FuncType.LOAD_ONLY,
     call_string='Player.onEvent',
@@ -213,7 +302,7 @@ class PlayerOnEvent(JMCFunction):
     def call(self) -> str:
         count = self.datapack.get_count(self.name)
         func = self.datapack.add_raw_private_function(
-            self.name, [f"scoreboard players reset @s {self.args['objective']}", self.args['function']], count=count)
+            self.name, [f"scoreboard players set @s {self.args['objective']} 0", self.args['function']], count=count)
         self.datapack.add_tick_command(
             f"execute as @a[scores={{{self.args['objective']}=1..}}] at @s run {func}")
         return ""
@@ -290,7 +379,7 @@ class TriggerSetup(JMCFunction):
                     run.append(
                         f'execute if score @s {obj} {DataPack.var_name} matches {num} run function {self.datapack.namespace}:{func}')
 
-        run.extend([f"scoreboard players reset @s {obj}",
+        run.extend([f"scoreboard players set @s {obj} 0",
                     f"scoreboard players enable @s {obj}"])
         self.datapack.add_raw_private_function(self.name, run, main_count)
 
@@ -338,13 +427,13 @@ class TriggerAdd(JMCFunction):
         if self.raw_args["function"].arg_type == ArgType.ARROW_FUNC:
             run = f"""execute as @a[scores={{{obj}=1..}}] at @s run {self.datapack.add_raw_private_function(self.name, [
                 func,
-                f"scoreboard players reset @s {obj}",
+                f"scoreboard players set @s {obj} 0",
                 f"scoreboard players enable @s {obj}"
                 ], obj)}"""
         else:
             run = f"""execute as @a[scores={{{obj}=1..}}] at @s run {self.datapack.add_raw_private_function(self.name, [
                 f"{self.datapack.namespace}:{func}",
-                f"scoreboard players reset @s {obj}",
+                f"scoreboard players set @s {obj} 0",
                 f"scoreboard players enable @s {obj}"
                 ], obj)}"""
         main_func.append(run)
@@ -396,7 +485,7 @@ class TimerAdd(JMCFunction):
                 f"""execute as {selector} if score @s {obj} matches 0 run {self.datapack.add_raw_private_function(
                     self.name,
                     [
-                        f"scoreboard players reset @s {obj}",
+                        f"scoreboard players set @s {obj} 0",
                         self.args["function"]
                     ],
                     count
@@ -413,7 +502,7 @@ class TimerAdd(JMCFunction):
         return ""
 
 
-@ func_property(
+@func_property(
     func_type=FuncType.LOAD_ONLY,
     call_string='Recipe.table',
     arg_type={
