@@ -1,3 +1,4 @@
+"""Untility for commands"""
 import ast
 import json
 import operator as op
@@ -38,7 +39,7 @@ def find_scoreboard_player_type(
     """
     if token.token_type != TokenType.KEYWORD:
         raise JMCSyntaxException(
-            f"Expected keyword", token, tokenizer)
+            "Expected keyword", token, tokenizer)
 
     if token.string.startswith(DataPack.VARIABLE_SIGN):
         return ScoreboardPlayer(player_type=PlayerType.VARIABLE, value=(
@@ -52,10 +53,9 @@ def find_scoreboard_player_type(
     if len(splits) == 1:
         if allow_integer:
             raise JMCSyntaxException(
-                f"Expected integer, variable, or objective:selector", token, tokenizer)
-        else:
-            raise JMCSyntaxException(
-                f"Expected variable or objective:selector", token, tokenizer)
+                "Expected integer, variable, or objective:selector", token, tokenizer)
+        raise JMCSyntaxException(
+            "Expected variable or objective:selector", token, tokenizer)
     if len(splits) > 2:
         raise JMCSyntaxException(
             "Scoreboard's player cannot contain more than 1 colon(:)", token, tokenizer)
@@ -64,6 +64,12 @@ def find_scoreboard_player_type(
         player_type=PlayerType.SCOREBOARD, value=(splits[0], splits[1]))
 
 
+class NumberType(Enum):
+    POSITIVE = "more than zero"
+    ZERO_POSITIVE = "more than or equal to zero"
+    NON_ZERO = "non-zero"
+
+
 class ArgType(Enum):
     ARROW_FUNC = "arrow(anonymous) function"
     JS_OBJECT = "JavaScript object (dictionary)"
@@ -113,7 +119,7 @@ class Arg:
         if verifier == ArgType.FUNC:
             if self.arg_type == ArgType.ARROW_FUNC:
                 return self
-            if self.arg_type == ArgType.KEYWORD:
+            if self.arg_type in {ArgType.KEYWORD, ArgType.SCOREBOARD}:
                 self.arg_type = ArgType._FUNC_CALL
                 return self
             raise JMCValueError(
@@ -152,8 +158,7 @@ def find_arg_type(token: Token, tokenizer: Tokenizer) -> ArgType:
     if token.token_type == TokenType.PAREN_CURLY:
         if re.match(r'^{\s*"', token.string) is not None:
             return ArgType.JSON
-        else:
-            return ArgType.JS_OBJECT
+        return ArgType.JS_OBJECT
     if token.token_type == TokenType.PAREN_SQUARE:
         return ArgType.LIST
 
@@ -232,12 +237,20 @@ def __eval(node):
     """
     if isinstance(node, ast.Num):  # <number>
         return node.n
-    elif isinstance(node, ast.BinOp):  # <left> <operator> <right>
+    if isinstance(node, ast.BinOp):  # <left> <operator> <right>
         return OPERATORS[type(node.op)](__eval(node.left), __eval(node.right))
-    elif isinstance(node, ast.UnaryOp):  # <operator> <operand> e.g., -1
+    if isinstance(node, ast.UnaryOp):  # <operator> <operand> e.g., -1
         return OPERATORS[type(node.op)](__eval(node.operand))
-    else:
-        raise TypeError(node)
+
+    raise TypeError(node)
+
+
+SIMPLE_JSON_TYPE = dict[
+    str,
+    str | bool | dict[
+        str, str | bool
+    ]
+]
 
 
 class FormattedText:
@@ -245,18 +258,20 @@ class FormattedText:
     Parse formatted text into raw json string
 
     :param raw_text: Text
-    :param token: Token
+    :param token: Token (only used for error)
     :param tokenizer: Tokenizer
     :param is_default_no_italic: Whether to set italic to False by default, defaults to False
     :return: Raw JSON string
 
-    .. Example::
-    >>> minecraft_formatted_text("&4&lName")
+    .. example::
+    >>> str(FormattedText("&4&lName", token, tokenizer))
     '{"text":"Name", "color":"red", "bold":true}'
+    >>> str(FormattedText("&<red,bold>Name", token, tokenizer, is_default_no_italic=True))
+    '{"text":"Name", "color":"red", "bold":true, "italic":false}'
     """
     __slots__ = ('raw_text', 'current_json', 'result',
                  'bracket_content', 'token', 'tokenizer',
-                 'current_color', 'is_default_no_italic')
+                 'current_color', 'is_default_no_italic', 'datapack')
 
     SIGN = "&"
     OPEN_BRACKET = "<"
@@ -287,27 +302,36 @@ class FormattedText:
     }
 
     def __init__(self, raw_text: str, token: Token,
-                 tokenizer: Tokenizer, *, is_default_no_italic: bool = False) -> None:
+                 tokenizer: Tokenizer, datapack: DataPack, *, is_default_no_italic: bool = False) -> None:
         self.raw_text = raw_text
         self.token = token
         self.tokenizer = tokenizer
+        self.datapack = datapack
         self.is_default_no_italic = is_default_no_italic
         self.bracket_content = ""
-        self.result: list[dict[str, str | bool]] = []
-        self.current_json: dict[str, str | bool] = {"text": ""}
+        self.result: list[SIMPLE_JSON_TYPE] = []
+        self.current_json: SIMPLE_JSON_TYPE = {"text": ""}
         self.current_color = ""
-        self.parse()
+        self.__parse()
 
-    def push(self) -> None:
+    def add_key(self, key: str, value: str | bool |
+                dict[str, str | bool]) -> None:
+        if self.result:
+            self.result[0][key] = value
+            return
+
+        self.result.append({key: value})
+
+    def __push(self) -> None:
         """
         Append current_json to result and reset it
         """
         if not self.current_json["text"]:
-            return None
+            return
         self.result.append(self.current_json)
         self.current_json = {"text": ""}
 
-    def parse_bracket(self) -> None:
+    def __parse_bracket(self) -> None:
         """
         Parse self.bracket_content and reset it
         """
@@ -365,13 +389,51 @@ class FormattedText:
                         f'Color({prop}) cannot be false', self.token, self.tokenizer)
                 continue
 
+            if prop.startswith('$'):
+                if 'selector' in self.current_json:
+                    raise JMCValueError(
+                        f'selector used with score in formatted text', self.token, self.tokenizer)
+                if 'score' in self.current_json:
+                    raise JMCValueError(
+                        f'score used twice in formatted text', self.token, self.tokenizer)
+                self.current_json['score'] = {
+                    "name": prop, "objective": self.datapack.var_name}
+                continue
+
+            if prop.count(':') == 1:
+                if 'selector' in self.current_json:
+                    raise JMCValueError(
+                        f'selector used with score in formatted text', self.token, self.tokenizer)
+                if 'score' in self.current_json:
+                    raise JMCValueError(
+                        f'score used twice in formatted text', self.token, self.tokenizer)
+                objective, name = prop.split(':')
+                self.current_json['score'] = {
+                    "name": objective, "objective": name}
+                continue
+
+            if prop.startswith('@'):
+                if 'score' in self.current_json:
+                    raise JMCValueError(
+                        f'score used with selector in formatted text', self.token, self.tokenizer)
+                if 'selector' in self.current_json:
+                    raise JMCValueError(
+                        f'selector used with selector in formatted text', self.token, self.tokenizer)
+                self.current_json['selector'] = prop
+                continue
+
             raise JMCValueError(
                 f"Unknown property '{prop}'", self.token, self.tokenizer)
 
         if 'color' not in self.current_json and self.current_color:
             self.current_json['color'] = self.current_color
 
-    def parse_code(self, char: str) -> None:
+        if 'score' in self.current_json or 'selector' in self.current_json:
+            del self.current_json['text']
+            self.result.append(self.current_json)
+            self.current_json = {"text": ""}
+
+    def __parse_code(self, char: str) -> None:
         """
         Parse color code
 
@@ -413,7 +475,7 @@ class FormattedText:
                 self.current_json['color'] = self.current_color
             self.current_json[prop] = True
 
-    def parse(self) -> None:
+    def __parse(self) -> None:
         """
         Parse raw_text
         """
@@ -424,7 +486,7 @@ class FormattedText:
             if is_bracket_format:
                 if char == self.CLOSE_BRACKET:
                     is_bracket_format = False
-                    self.parse_bracket()
+                    self.__parse_bracket()
                     continue
                 self.bracket_content += char
                 continue
@@ -438,28 +500,37 @@ class FormattedText:
                     if isinstance(self.current_json["text"], str):
                         self.current_json["text"] += char
                         continue
-                self.parse_code(char)
+                self.__parse_code(char)
                 continue
 
             if char == self.SIGN:
-                self.push()
+                self.__push()
                 is_expect_color_code = True
                 continue
 
             if isinstance(self.current_json["text"], str):
                 self.current_json["text"] += char
 
-        self.push()
+        if is_bracket_format:
+            raise JMCValueError(
+                f"'<' was never closed in formatted text", self.token, self.tokenizer)
+        self.__push()
 
         if is_expect_color_code:
             raise JMCValueError(
                 f"Unexpected trailing '{self.SIGN}'", self.token, self.tokenizer, suggestion=f"Remove last '{self.SIGN}'")
 
+    def __bool__(self) -> bool:
+        return bool(self.result) and ("text" in self.result[0])
+
     def __str__(self) -> str:
         if len(self.result) == 1:
-            if 'italic' not in self.result[0]:
+            if self.is_default_no_italic and 'italic' not in self.result[0]:
                 self.result[0]['italic'] = False
             return json.dumps(self.result[0])
+
+        if not self.result:
+            return ''
         return json.dumps(
             [{"italic": False} if self.is_default_no_italic else ""] + self.result)
 
