@@ -3,6 +3,7 @@ from json import loads, JSONDecodeError, dumps
 from typing import TYPE_CHECKING
 
 
+from .header import Header
 from .exception import JMCDecodeJSONError, JMCFileNotFoundError, JMCSyntaxException, MinecraftSyntaxWarning
 from .tokenizer import Tokenizer, Token, TokenType
 from .datapack import DataPack, Function
@@ -21,12 +22,11 @@ JSON_FILE_TYPES = [
     "advancements",
     "dimension",
     "dimension_type",
-    "functions",
     "loot_tables",
     "predicates",
     "recipes",
+    "item_modifiers",
     "structures",
-    "tags",
     "worldgen/biome",
     "worldgen/configured_carver",
     "worldgen/configured_feature",
@@ -59,11 +59,18 @@ class Lexer:
         """JMC configuration"""
         self.datapack = DataPack(config.namespace, self)
         """Datapack object"""
+        self.datapack.functions[self.datapack.load_name] = Function()
         self.parse_file(Path(self.config.target), _test_file, is_load=True)
 
-        logger.debug(f"Load Function")
-        self.datapack.functions[self.datapack.load_name] = Function(
-            self.parse_load_func_content(programs=self.datapack.load_function))
+        logger.debug("Load Function")
+        self.parse_current_load()
+
+    def parse_current_load(self):
+        """Parse current load function that's in self.datapack.load_function and clear it"""
+        if self.datapack.load_function:
+            self.datapack.functions[self.datapack.load_name].extend(
+                self.parse_load_func_content(programs=self.datapack.load_function))
+            self.datapack.load_function = []
 
     def parse_file(self, file_path: Path, _test_file: str | None = None,
                    is_load=False) -> None:
@@ -85,9 +92,9 @@ class Lexer:
             try:
                 with file_path.open('r') as file:
                     raw_string = file.read()
-            except FileNotFoundError:
+            except FileNotFoundError as error:
                 raise JMCFileNotFoundError(
-                    f"JMC file not found: {file_path.resolve().as_posix()}")
+                    f"JMC file not found: {file_path.resolve().as_posix()}") from error
         else:
             raw_string = _test_file
         tokenizer = Tokenizer(raw_string, file_path_str)
@@ -97,13 +104,14 @@ class Lexer:
         for command in tokenizer.programs:
             if command[0].string == 'function' and len(command) == 4:
                 self.parse_func(tokenizer, command, file_path_str)
+                self.parse_current_load()
             elif command[0].string == 'function' and len(command) != 2:
                 raise JMCSyntaxException(
                     f"'function' expect 1(Vanilla syntax) or 3(JMC syntax) arguments (got {len(command)-1})",
                     command[0],
                     tokenizer)
             elif command[0].string == 'new':
-                self.parse_new(tokenizer, command, file_path_str)
+                self.parse_new(tokenizer, command)
             elif command[0].string == 'class':
                 self.parse_class(tokenizer, command, file_path_str)
             elif command[0].string == '@import':
@@ -116,6 +124,21 @@ class Lexer:
                 if len(command) > 2:
                     raise JMCSyntaxException(
                         "Unexpected token", command[1], tokenizer, display_col_length=False)
+                if command[1].string.endswith(
+                        "/*") or command[1].string.endswith("\\*"):
+                    try:
+                        folder = Path(command[1].string[:-2])
+                    except Exception as error:
+                        raise JMCSyntaxException(
+                            f"Unexpected invalid path ({command[1].string})", command[1], tokenizer) from error
+                    if not folder.is_dir():
+                        raise JMCFileNotFoundError(
+                            f"Directory(folder) not found: {folder.resolve().as_posix()}")
+
+                    new_paths = folder.glob("*.jmc")
+                    for new_path in new_paths:
+                        self.parse_file(file_path=new_path)
+                    return
                 try:
                     new_path = Path(
                         (file_path.parent / command[1].string).resolve()
@@ -125,14 +148,16 @@ class Lexer:
                             (file_path.parent /
                              (command[1].string + '.jmc')).resolve()
                         )
-                except Exception:
+                except Exception as error:
                     raise JMCSyntaxException(
-                        f"Unexpected invalid path ({command[1].string})", command[1], tokenizer)
+                        f"Unexpected invalid path ({command[1].string})", command[1], tokenizer) from error
                 self.parse_file(file_path=new_path)
             else:
-                if not is_load:
-                    raise JMCSyntaxException(
-                        f"Command({command[0].string}) found inside non-load file (Not main JMC file)", command[1], tokenizer, suggestion="Consider putting it in a new function")
+                # if not is_load:
+                #     raise JMCSyntaxException(
+                # f"Command({command[0].string}) found inside non-load file
+                # (Not main JMC file)", command[1], tokenizer,
+                # suggestion="Consider putting it in a new function")
 
                 self.datapack.load_function.append(command)
 
@@ -157,10 +182,10 @@ class Lexer:
         if command[1].token_type != TokenType.KEYWORD:
             raise JMCSyntaxException(
                 "Expected keyword(function's name)", command[1], tokenizer)
-        elif command[2].string != '()':
+        if command[2].string != '()':
             raise JMCSyntaxException(
-                f"Expected (", command[2], tokenizer, display_col_length=False)
-        elif command[3].token_type != TokenType.PAREN_CURLY:
+                "Expected empty round parenthesis", command[2], tokenizer)
+        if command[3].token_type != TokenType.PAREN_CURLY:
             raise JMCSyntaxException(
                 "Expected {", command[3], tokenizer, display_col_length=False)
 
@@ -170,26 +195,30 @@ class Lexer:
                 f"Function({func_path}) may override private function of JMC", command[1], tokenizer, suggestion=f"Please avoid starting function's path with {DataPack.private_name}")
         logger.debug(f"Function: {func_path}")
         func_content = command[3].string[1:-1]
+        if func_path == self.datapack.load_name:
+            raise JMCSyntaxException(
+                "Load function is defined", command[1], tokenizer)
         if func_path in self.datapack.functions:
+            old_function_token, old_function_tokenizer = self.datapack.defined_file_pos[
+                func_path]
             raise JMCSyntaxException(
-                f"Duplicate function declaration({func_path})", command[1], tokenizer, display_col_length=False)
-        elif func_path == self.datapack.load_name:
-            raise JMCSyntaxException(
-                "Load function is defined", command[1], tokenizer, display_col_length=False)
-        elif func_path == self.datapack.private_name:
+                f"Duplicate function declaration({func_path})", command[1], tokenizer,
+                suggestion=f"This function was already defined at line {old_function_token.line} col {old_function_token.col} in {old_function_tokenizer.file_path}")
+        if func_path == self.datapack.private_name:
             raise JMCSyntaxException(
                 "Private function is defined", command[1], tokenizer, display_col_length=False)
+        self.datapack.defined_file_pos[func_path] = (command[1], tokenizer)
         self.datapack.functions[func_path] = Function(self.parse_func_content(
             func_content, file_path_str, line=command[3].line, col=command[3].col, file_string=tokenizer.file_string))
 
     def parse_new(self, tokenizer: Tokenizer,
-                  command: list[Token], file_path_str: str, prefix: str = ''):
+                  command: list[Token], prefix: str = ''):
         """
         Parse a new json definition in form of list of token
 
         :param tokenizer: Tokenizer
         :param command: List of token inside a new json definition
-        :param file_path_str: File path to current JMC function as string
+        # :param file_path_str: File path to current JMC function as string
         :param prefix: Prefix of json(for Class feature), defaults to ''
         :raises JMCSyntaxException: JSON file's type is not given
         :raises JMCSyntaxException: JSON file's type is not a keyword token
@@ -213,13 +242,13 @@ class Lexer:
                 "Expected keyword(JSON file's type)", command[1], tokenizer)
         if len(command) < 3:
             raise JMCSyntaxException(
-                "Expected JSON file's path in bracket", command[1], tokenizer, col_length=True)
+                "Expected round parenthesis(JSON file's path)", command[1], tokenizer, col_length=True)
         if command[2].string == '()':
             raise JMCSyntaxException(
-                "Expected JSON file's path in bracket", command[2], tokenizer)
+                "Expected JSON file's path in the bracket", command[1], tokenizer)
         if command[2].token_type != TokenType.PAREN_ROUND:
             raise JMCSyntaxException(
-                "Expected (", command[2], tokenizer)
+                "Expected (", command[1], tokenizer)
         if len(command) < 4:
             raise JMCSyntaxException(
                 "Expected {", command[2], tokenizer, col_length=True)
@@ -229,14 +258,22 @@ class Lexer:
 
         json_type = convention_jmc_to_mc(
             command[1], tokenizer, is_make_lower=False)
-        if json_type not in JSON_FILE_TYPES:
+
+        if json_type not in JSON_FILE_TYPES and not json_type.startswith(
+                "tags/"):
             raise MinecraftSyntaxWarning(
                 f"Unrecognized JSON file's type({json_type})", command[2], tokenizer
             )
 
-        json_path = json_type + '/' + prefix + \
-            convention_jmc_to_mc(
-                command[2], tokenizer, is_make_lower=False, substr=(1, -1))
+        json_name = prefix + convention_jmc_to_mc(
+            command[2], tokenizer, is_make_lower=False, substr=(1, -1))
+
+        if Header().is_override_minecraft and json_name.startswith('minecraft/'):
+            # len('minecraft/') = 10
+            json_path = 'minecraft/' + json_type + '/' + json_name[10:]
+        else:
+            json_path = json_type + '/' + json_name
+
         if not json_path.islower():
             raise MinecraftSyntaxWarning(
                 f"Uppercase letter found in JSON file's path({json_path})", command[
@@ -249,16 +286,20 @@ class Lexer:
         logger.debug(f"JSON: {json_type}({json_path})")
         json_content = command[3].string
         if json_path in self.datapack.jsons:
+            old_json_token, old_json_tokenizer = self.datapack.defined_file_pos[
+                json_path]
             raise JMCSyntaxException(
-                f"Duplicate JSON({json_path})", command[2], tokenizer)
+                f"Duplicate JSON({json_path})", command[2], tokenizer,
+                suggestion=f"This json was already defined at line {old_json_token.line} col {old_json_token.col} in {old_json_tokenizer.file_path}")
 
         try:
             json: dict[str, str] = loads(json_content)
         except JSONDecodeError as error:
-            raise JMCDecodeJSONError(error, command[3], tokenizer)
+            raise JMCDecodeJSONError(error, command[3], tokenizer) from error
         if not json:
             raise JMCSyntaxException(
                 "JSON content cannot be empty", command[3], tokenizer)
+        self.datapack.defined_file_pos[json_path] = (command[1], tokenizer)
         self.datapack.jsons[json_path] = json
 
     def parse_class(self, tokenizer: Tokenizer,
@@ -319,6 +360,16 @@ class Lexer:
     def parse_func_content(self,
                            func_content: str, file_path_str: str,
                            line: int, col: int, file_string: str) -> list[str]:
+        """
+        Parse function's content
+
+        :param func_content: String containing function's content
+        :param file_path_str: File path to current JMC function as string
+        :param line: Current line in current JMC function
+        :param col: Current column in current JMC function
+        :param file_string: Content of current JMC function
+        :return: List of commands(string)
+        """
         tokenizer = Tokenizer(func_content, file_path_str,
                               line=line, col=col, file_string=file_string)
         programs = tokenizer.programs
@@ -357,7 +408,7 @@ class Lexer:
             if command[0].string == 'function' and len(command) == 4:
                 self.parse_func(tokenizer, command, file_path_str, prefix)
             elif command[0].string == 'new':
-                self.parse_new(tokenizer, command, file_path_str, prefix)
+                self.parse_new(tokenizer, command, prefix)
             elif command[0].string == 'class':
                 self.parse_class(tokenizer, command, file_path_str, prefix)
             elif command[0].string == '@import':
@@ -445,12 +496,12 @@ class Lexer:
         """
         if len(token.string) == 2:
             return token.string
-        open = token.string[0]
+        open_ = token.string[0]
         close = token.string[-1]
         tokenizer = Tokenizer(token.string[1:-1], tokenizer.file_path, token.line,
-                              token.col + 1, tokenizer.file_string, expect_semicolon=False)
+                              token.col + 1, tokenizer.file_string, expect_semicolon=False, allow_semicolon=token.token_type == TokenType.PAREN_SQUARE)
         string = ""
-        if open == '{' and tokenizer.programs[0][0].token_type == TokenType.STRING:
+        if open_ == '{' and tokenizer.programs[0][0].token_type == TokenType.STRING:
             is_nbt = False
         for token_ in tokenizer.programs[0]:
             if token_.token_type == TokenType.PAREN_ROUND:
@@ -472,4 +523,4 @@ class Lexer:
                 _string = token_.string
             string += _string
 
-        return open + string + close
+        return open_ + string + close
